package com.huigou.topsun.sap.permit.application.impl;

import com.huigou.cache.DictUtil;
import com.huigou.data.query.model.QueryDescriptor;
import com.huigou.data.query.model.QueryModel;
import com.huigou.topsun.base.coderule.application.CodeRuleApplication;

import com.huigou.topsun.sap.fixedAssets.domain.SapRetirement;
import com.huigou.topsun.sap.fixedAssets.domain.SapRetirementItem;

import com.huigou.topsun.sap.permit.application.SapPermitApplication;
import com.huigou.topsun.sap.permit.application.SapPermitItemApplication;
import com.huigou.topsun.sap.permit.domain.SapPermit;
import com.huigou.topsun.sap.permit.domain.SapPermitItem;
import com.huigou.topsun.sap.permit.domain.query.SapPermitQueryRequest;
import com.huigou.topsun.sap.permit.repository.SapPermitItemRepository;
import com.huigou.topsun.sap.permit.repository.SapPermitRepository;
import com.huigou.topsun.sap.suppliers.domain.SuppliersInfo;
import com.huigou.uasp.bmp.common.BizBillStatus;
import com.huigou.uasp.bmp.doc.attachment.application.AttachmentApplication;
import com.huigou.uasp.bpm.FlowBroker;
import com.huigou.util.ClassHelper;
import org.activiti.engine.delegate.DelegateExecution;
import org.activiti.engine.delegate.DelegateTask;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;

/**
 * @Auther: Lxh
 * @Date: 2024/03/26/11:35
 * @Description:
 */
@Service("sapPermitApplication")
public class SapPermitApplicationImpl extends FlowBroker implements SapPermitApplication {
    @Autowired
    private SapPermitRepository sapPermitRepository;
    @Autowired
    private SapPermitItemRepository sapPermitItemRepository;
    @Autowired
    private SapPermitItemApplication sapPermitItemApplication;

   // @Autowired
   // private SapPermitItemRepository sapPermitItemRepository;
    @Autowired
    private AttachmentApplication attachmentApplication;
    @Autowired
    private CodeRuleApplication codeRuleApplication;
    @Override
    public Map<String, Object> getPermitList(SapPermitQueryRequest queryRequest) {
        QueryDescriptor queryDescriptor = this.sqlExecutorDao.getQuery(QUERY_XML_FILE_PATH, "sapPermit");
        QueryModel queryModel = this.sqlExecutorDao.getQueryModel(queryDescriptor, queryRequest);
        if (queryRequest.getStatus() != null) {
            queryModel.addCriteria(" and t.status =:status");
            queryModel.putParam("status", queryRequest.getStatus());
        }
        if (queryRequest.getBillCode() != null) {
            queryModel.addCriteria(" and t.bill_code =:billCode");
            queryModel.putParam("billCode", queryRequest.getBillCode());
        }
        if (queryRequest.getReason() != null) {
            queryModel.addCriteria(" and t.reason like:reason");
            queryModel.putLikeParam("reason", queryRequest.getReason());
        }
        queryModel.putDictionary("status", DictUtil.getDictionary("bizBillStatus"));
        queryModel.addCriteria(" order by bill_code desc ");
        Map<String, Object> map = this.sqlExecutorDao.executeSlicedQuery(queryModel);
        return map;
    }

    @Override
    public SapPermit getPermitById(String id) {
        return sapPermitRepository.findOne(id);
    }

    @Override
    public void deleteByIds(List<String> ids) {
        ids.forEach(id->{
             sapPermitRepository.delete(id);
            List<SapPermitItem> list=sapPermitItemRepository.findBySapPermitId(id);
            for (SapPermitItem sapPermitItem:list){
                sapPermitItemRepository.delete(sapPermitItem);
            }
        });
    }

    @Override
    protected void onBeforeComplete(DelegateTask delegateTask) {
        super.onBeforeComplete(delegateTask);
        String bizId = delegateTask.getExecution().getProcessBusinessKey();
        if (this.isApplyProcUnit(delegateTask) && this.getApprovalParameter().isAdvanceProcessAction()) {
            SapPermit sapPermit = sapPermitRepository.findOne(bizId);
            sapPermit.setStatusId(BizBillStatus.APPROVING.getId());
            sapPermitRepository.save(sapPermit);
        }
    }

    @Override
    protected void onEnd(DelegateExecution delegateExecution) {
        super.onEnd(delegateExecution);
        String bizId = delegateExecution.getProcessBusinessKey();
        BizBillStatus status = approvePassed() ? BizBillStatus.COMPLETED : BizBillStatus.ABORTED;
        SapPermit Permit = sapPermitRepository.findOne(bizId);
        Permit.setStatusId(status.getId());
        sapPermitRepository.save(Permit);
    }

    @Override
    protected void onAbortProcessInstance(DelegateExecution delegateExecution) {
        super.onAbortProcessInstance(delegateExecution);
        String bizId = delegateExecution.getProcessBusinessKey();
        SapPermit Permit = sapPermitRepository.findOne(bizId);
        Permit.setStatusId(BizBillStatus.ABORTED.getId());
        sapPermitRepository.save(Permit);
    }

    @Override
    protected void onBack(DelegateTask delegateTask, String destActivityId) {
        super.onBack(delegateTask, destActivityId);
        if ("Apply".equalsIgnoreCase(destActivityId)) {
            String bizId = delegateTask.getExecution().getProcessBusinessKey();
            SapPermit Permit = sapPermitRepository.findOne(bizId);
            Permit.setStatusId(BizBillStatus.APPLYING.getId());
            sapPermitRepository.save(Permit);
        }
    }

    @Override
    protected Map<String, Object> getProcessBizParams(String bizId) {
        return ClassHelper.toMap(sapPermitRepository.getOne(bizId));
    }

    @Override
    protected String saveBizAndApprovalData() {
        super.saveBizAndApprovalData();
        SapPermit permit = getBizEntity(SapPermit.class);
        if (permit.isNew()) {
            permit.setStatusId(BizBillStatus.APPLYING.getId());
        } else {
            permit = (SapPermit) commonDomainService.loadAndFillinProperties(permit);
        }
        permit = sapPermitRepository.save(permit);
        List<SapPermitItem> items = getBizEntities(SapPermitItem.class, "items");
        if (items.size()<1){
           throw new IllegalArgumentException("没有填写携带物品信息!");
        }
        sapPermitItemApplication.saveSapPermitItems(permit.getId(),items);
        //更新附件ID为主键ID
//        if (!(Permit.getId().equals(Permit.getAttbizId()))) {
//            List<Attachment> attachmentList = attachmentApplication.queryAttachments("sapRetirementInfo", Permit.getAttbizId());
//            for (Attachment attachment : attachmentList) {
//                attachment.setBizId(Permit.getId());
//                attachmentApplication.saveAttachment(attachment);
//            }
//        }
        return permit.getId();
    }



}
